require("vanilla-cookieconsent")
var ServerConnector = require("./ServerConnector")
var ConfigurationType = require("./ConfigurationType");
var Promise = require("bluebird");

var cc;
var cookiePolicyUrl

/**
 *
 * @returns {Promise}
 */
function insertGoogleAnalyticsCode() {
  return ServerConnector.getConfigurationParam(ConfigurationType.GOOGLE_ANALYTICS_IDENTIFIER).then(function (identifier) {
    if (identifier === "" || identifier === undefined || identifier === null) {
      return Promise.resolve();
    } else {
      global._gaq = global._gaq || [];
      global._gaq.push(['_setAccount', identifier]);
      global._gaq.push(['_trackPageview']);

      (function () {
        var ga = document.createElement('script');
        ga.type = 'text/javascript';
        ga.async = true;
        ga.src = ('https:' === document.location.protocol ? 'https://ssl' : 'http://www')
          + '.google-analytics.com/ga.js';
        var s = document.getElementsByTagName('script')[0];
        s.parentNode.insertBefore(ga, s);
      })();
      return Promise.resolve();
    }
  });
}


window.addEventListener('load', function () {
  return new Promise(function (resolve) {
    setTimeout(resolve, 5000) //wait for basic resources to load (otherwise we might load data before session/credentials are defined)
  }).then(function () {
    return ServerConnector.getConfiguration();
  }).then(function (configuration) {

    var option = configuration.getOption(ConfigurationType.COOKIE_POLICY_URL);
    if (option !== undefined && option != null) {
      cookiePolicyUrl = option.getValue();
    }
    var cookieConfig = {
      current_lang: 'en',
      autoclear_cookies: true,                   // default: false
      page_scripts: true,                        // default: false

      // mode: 'opt-in'                          // default: 'opt-in'; value: 'opt-in' or 'opt-out'
      // delay: 0,                               // default: 0
      // auto_language: '',                      // default: null; could also be 'browser' or 'document'
      autorun: false,                          // default: true
      // force_consent: true,                   // default: false
      // hide_from_bots: false,                  // default: false
      // remove_cookie_tables: false             // default: false
      // cookie_name: 'cc_cookie',               // default: 'cc_cookie'
      // cookie_expiration: 182,                 // default: 182 (days)
      // cookie_necessary_only_expiration: 182   // default: disabled
      // cookie_domain: location.hostname,       // default: current domain
      // cookie_path: '/',                       // default: root
      // cookie_same_site: 'Lax',                // default: 'Lax'
      // use_rfc_cookie: false,                  // default: false
      // revision: 0,                            // default: 0

      onFirstAction: function (user_preferences, cookie) {
        // callback triggered only once on the first accept/reject action
      },

      onAccept: function (cookie) {
        // callback triggered on the first accept/reject action, and after each page load
        if (cookie.level.indexOf('analytics') >= 0) {
          return insertGoogleAnalyticsCode();
        }
      },

      onChange: function (cookie, changed_categories) {
        // callback triggered when user changes preferences after consent has already been given
      },

      languages: {
        'en': {
          consent_modal: {
            title: 'We use cookies!',
            description: 'Minerva platform uses essential cookies to ensure its proper operation and tracking cookies to understand how you interact with it. The latter will be set only after consent. <button type="button" data-cc="c-settings" class="cc-link">Let me choose</button>',
            primary_btn: {
              text: 'Accept all',
              role: 'accept_all'              // 'accept_selected' or 'accept_all'
            },
            secondary_btn: {
              text: 'Reject',
              role: 'accept_necessary'        // 'settings' or 'accept_necessary'
            }
          },
          settings_modal: {
            title: 'Cookie preferences',
            save_settings_btn: 'Save settings',
            accept_all_btn: 'Accept all',
            reject_all_btn: 'Reject',
            close_btn_label: 'Close',
            blocks: [
              {
                title: 'Cookie usage 📢',
                description: 'I use cookies to ensure the basic functionalities of the website and to enhance your online experience. For more details relative to cookies and other sensitive data, please read the full <a href="' + cookiePolicyUrl + '" class="cc-link" target="minerva_cookie">privacy policy</a>.'
              }, {
                title: 'Strictly necessary cookies',
                description: 'These cookies are essential for the proper functioning of the website. Without these cookies, the website would not work properly',
                toggle: {
                  value: 'necessary',
                  enabled: true,
                  readonly: true          // cookie categories with readonly=true are all treated as "necessary cookies"
                }
              }, {
                title: 'Performance and Analytics cookies',
                description: 'These cookies allow the website to remember the choices you have made in the past',
                toggle: {
                  value: 'analytics',     // your cookie category
                  enabled: true,
                  readonly: false
                }
              }
            ]
          }
        }
      }
    };
    var identifier = configuration.getOption(ConfigurationType.GOOGLE_ANALYTICS_IDENTIFIER).getValue();
    if (identifier === null || identifier === undefined || identifier === "") {
      cookieConfig.languages.en.consent_modal.description = "Minerva platform uses essential cookies to ensure its " +
        "proper operation. For any queries in relation to our policy on cookies and your choices, please " +
        "<a class='cc-link' href='" + cookiePolicyUrl + "' target='minerva_cookie'>read here</a>."
      cookieConfig.languages.en.settings_modal.blocks = [{
        title: 'More information',
        description: 'For any queries in relation to our policy on cookies and your choices, please <a class="cc-link" href="' + cookiePolicyUrl + '" target="minerva_cookie">read here</a>.'
      }];
      cookieConfig.languages.en.consent_modal.primary_btn = {
        text: 'OK',
        role: 'accept_all'
      };
      cookieConfig.languages.en.consent_modal.secondary_btn = undefined;
    }

    // obtain plugin
    cc = initCookieConsent();

    // run plugin with your configuration
    cc.run(cookieConfig);
  });
});

module.exports = {
  showCookieConsent: function () {
    if (cookiePolicyUrl !== undefined && cookiePolicyUrl !== null && cookiePolicyUrl !== "") {
      cc.show();
    } else {
      console.log("Cookie policy url is not defined");
    }
  }
};
