package lcsb.mapviewer.persist.dao;

import java.util.Calendar;
import java.util.List;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Root;

import org.springframework.stereotype.Repository;

import lcsb.mapviewer.model.job.MinervaJob;
import lcsb.mapviewer.model.job.MinervaJobStatus;

@Repository
public class MinervaJobDao extends BaseDao<MinervaJob> {

  public MinervaJobDao() {
    super(MinervaJob.class);
  }

  public List<MinervaJob> getRunning() {
    return getElementsByParameter("jobStatus", MinervaJobStatus.RUNNING);
  }

  public long getCount(final MinervaJobStatus status) {
    CriteriaBuilder builder = getSession().getCriteriaBuilder();
    CriteriaQuery<Long> criteria = builder.createQuery(Long.class);
    Root<MinervaJob> root = criteria.from(MinervaJob.class);
    criteria.where(builder.equal(root.get("jobStatus"), status));
    criteria.select(builder.count(root));
    return getSession().createQuery(criteria).getSingleResult();
  }

  public List<MinervaJob> getPending() {
    return getPending(Integer.MAX_VALUE);
  }

  public List<MinervaJob> getPending(final int max) {
    List<MinervaJob> list = getSession()
        .createQuery(" from " + MinervaJob.class.getSimpleName() + " where jobStatus = :job_status "
            + " order by priority, id",
            MinervaJob.class)
        .setMaxResults(max)
        .setParameter("job_status", MinervaJobStatus.PENDING).list();

    return list;
  }

  public void deleteBefore(final MinervaJobStatus pending, final Calendar beforeDate) {
    String query = "DELETE from " + this.getClazz().getSimpleName() + " where jobStatus  = :job_status and jobFinished < :before_date";
    getSession().createQuery(query)
        .setParameter("job_status", pending)
        .setParameter("before_date", beforeDate)
        .executeUpdate();

  }

}
