package lcsb.mapviewer.persist.dao;

import static org.junit.Assert.assertEquals;

import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;

import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

import lcsb.mapviewer.model.job.MinervaJob;
import lcsb.mapviewer.model.job.MinervaJobParameters;
import lcsb.mapviewer.model.job.MinervaJobPriority;
import lcsb.mapviewer.model.job.MinervaJobStatus;
import lcsb.mapviewer.model.job.MinervaJobType;
import lcsb.mapviewer.persist.PersistTestFunctions;

public class MinervaJobDaoTest extends PersistTestFunctions {

  private class MockMinervaJob extends MinervaJobParameters {

    @Override
    protected void assignData(final Map<String, Object> jobParameters) {
    }

    @Override
    protected Map<String, Object> getJobParameters() {
      return new HashMap<>();
    }

  }

  @Autowired
  private MinervaJobDao minervaJobDao;

  @Test
  public void testGetPending() {
    MinervaJob job = new MinervaJob(MinervaJobType.REFRESH_CACHE, MinervaJobPriority.LOWEST, new MockMinervaJob());
    job.setJobStatus(MinervaJobStatus.PENDING);
    minervaJobDao.add(job);

    job = new MinervaJob(MinervaJobType.REFRESH_CACHE, MinervaJobPriority.LOWEST, new MockMinervaJob());
    job.setJobStatus(MinervaJobStatus.FAILED);
    minervaJobDao.add(job);

    assertEquals(1, minervaJobDao.getPending().size());
    assertEquals(1, minervaJobDao.getCount(MinervaJobStatus.PENDING));
    assertEquals(0, minervaJobDao.getCount(MinervaJobStatus.INTERRUPTED));
  }

  @Test
  public void testGetCount() {
    MinervaJob job = new MinervaJob(MinervaJobType.REFRESH_CACHE, MinervaJobPriority.LOWEST, new MockMinervaJob());
    job.setJobStatus(MinervaJobStatus.PENDING);
    minervaJobDao.add(job);

    job = new MinervaJob(MinervaJobType.REFRESH_CACHE, MinervaJobPriority.LOWEST, new MockMinervaJob());
    job.setJobStatus(MinervaJobStatus.FAILED);
    minervaJobDao.add(job);

    assertEquals(1, minervaJobDao.getCount(MinervaJobStatus.PENDING));
    assertEquals(0, minervaJobDao.getCount(MinervaJobStatus.INTERRUPTED));
  }

  @Test
  public void testRemoveBefore() {
    MinervaJob job = new MinervaJob(MinervaJobType.REFRESH_CACHE, MinervaJobPriority.LOWEST, new MockMinervaJob());
    job.setJobStatus(MinervaJobStatus.DONE);
    minervaJobDao.add(job);

    assertEquals(1, minervaJobDao.getCount(MinervaJobStatus.DONE));

    Calendar yesterday = Calendar.getInstance();
    yesterday.add(Calendar.DATE, -1);

    minervaJobDao.deleteBefore(MinervaJobStatus.DONE, yesterday);
    assertEquals(1, minervaJobDao.getCount(MinervaJobStatus.DONE));

    Calendar tomorrow = Calendar.getInstance();
    tomorrow.add(Calendar.DATE, 1);

    minervaJobDao.deleteBefore(MinervaJobStatus.DONE, tomorrow);
    assertEquals(0, minervaJobDao.getCount(MinervaJobStatus.DONE));

  }

}
