package lcsb.mapviewer.persist.dao.graphics;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

import java.awt.Color;
import java.awt.geom.Point2D;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

import lcsb.mapviewer.common.Configuration;
import lcsb.mapviewer.model.graphics.ArrowType;
import lcsb.mapviewer.model.graphics.LineType;
import lcsb.mapviewer.model.graphics.PolylineData;
import lcsb.mapviewer.persist.PersistTestFunctions;

public class PolylineDataTest extends PersistTestFunctions {

  @Autowired
  protected PolylineDao polylineDao;

  @Before
  public void setUp() throws Exception {
  }

  @After
  public void tearDown() throws Exception {
  }

  @Test
  public void test() {
    PolylineData pd = new PolylineData(new Point2D.Double(1, 1), new Point2D.Double(2, 3));
    pd.getBeginAtd().setArrowType(ArrowType.BLANK_CROSSBAR);
    pd.getBeginAtd().setArrowLineType(LineType.DASHED);
    pd.getBeginAtd().setLen(102);
    pd.setColor(Color.CYAN);
    pd.setType(LineType.SOLID);
    pd.setWidth(2);
    pd.addLine(new Point2D.Double(2, 3), new Point2D.Double(10, 11));
    polylineDao.add(pd);
    assertNotNull(pd.getId());
    PolylineData pd2 = polylineDao.getById(pd.getId());
    assertNotNull(pd2);

    assertEquals(ArrowType.BLANK_CROSSBAR, pd2.getBeginAtd().getArrowType());
    assertEquals(LineType.DASHED, pd2.getBeginAtd().getArrowLineType());
    assertEquals(102, pd2.getBeginAtd().getLen(), Configuration.EPSILON);
    assertEquals(Color.CYAN, pd2.getColor());
    assertEquals(LineType.SOLID, pd2.getType());
    assertEquals(2, pd2.getWidth(), Configuration.EPSILON);
    assertEquals(pd2.getLines().get(0).getP1(), new Point2D.Double(1, 1));
    assertEquals(pd2.getLines().get(0).getP2(), new Point2D.Double(2, 3));
    assertEquals(pd2.getLines().get(1).getP1(), new Point2D.Double(2, 3));
    assertEquals(pd2.getLines().get(1).getP2(), new Point2D.Double(10, 11));

    polylineDao.delete(pd);
    pd2 = polylineDao.getById(pd.getId());
    assertNull(pd2);
  }

}
