package lcsb.mapviewer.api.projects.models.units;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import lcsb.mapviewer.api.BaseController;
import lcsb.mapviewer.api.QueryException;
import lcsb.mapviewer.model.map.kinetics.SbmlUnit;
import lcsb.mapviewer.services.ObjectNotFoundException;
import lcsb.mapviewer.services.interfaces.IModelService;

@RestController
@RequestMapping(
    value = {
        "/minerva/api/projects/{projectId}/models/{modelId}/units",
    },
    produces = MediaType.APPLICATION_JSON_VALUE)
public class UnitsController extends BaseController {

  private IModelService modelService;

  @Autowired
  public UnitsController(final IModelService modelService) {
    this.modelService = modelService;
  }

  @PreAuthorize("hasAnyAuthority('IS_ADMIN', 'READ_PROJECT:' + #projectId)")
  @GetMapping(value = "/{unitId}")
  public SbmlUnit getUnit(
      final @PathVariable(value = "projectId") String projectId,
      final @PathVariable(value = "modelId") String modelId,
      final @PathVariable(value = "unitId") Integer unitId) throws QueryException, ObjectNotFoundException {
    for (final SbmlUnit unit : modelService.getUnitsForModel(projectId, modelId)) {
      if (unit.getId() == unitId) {
        return unit;
      }
    }
    throw new ObjectNotFoundException("Unit with given id doesn't exist");
  }

  @PreAuthorize("hasAnyAuthority('IS_ADMIN', 'READ_PROJECT:' + #projectId)")
  @GetMapping(value = "/")
  public List<SbmlUnit> getUnits(
      final @PathVariable(value = "projectId") String projectId,
      final @PathVariable(value = "modelId") String modelId) throws QueryException, ObjectNotFoundException {
    return modelService.getUnitsForModel(projectId, modelId);
  }

}