package lcsb.mapviewer.web.bean.utils;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 * This filter disables caching for API queries.
 * 
 * @author Piotr Gawron
 *
 */
public class CacheFilter implements Filter {
  /**
   * Default class logger.
   */
  @SuppressWarnings("unused")
  private final Logger logger = LogManager.getLogger();

  @Override
  public void init(final FilterConfig config) throws ServletException {
  }

  @Override
  public void doFilter(final ServletRequest req, final ServletResponse res, final FilterChain chain)
      throws IOException, ServletException {

    boolean disableCache = true;

    if (req instanceof HttpServletRequest) {
      String url = ((HttpServletRequest) req).getRequestURL().toString();
      if (url != null && url.contains("minerva/resources")) {
        disableCache = false;
      }
    }

    HttpServletResponse response = (HttpServletResponse) res;
    if (disableCache) {
      // caching on Safari
      response.addHeader("Vary", "*");
      // generic cache prevent mechanism
      response.addHeader("Cache-Control", "no-cache, no-store, must-revalidate");
      response.addHeader("Pragma", "no-cache");
      response.addHeader("Expires", "0");
    }
    chain.doFilter(req, response);
  }

  @Override
  public void destroy() {
  }

}
