package lcsb.mapviewer.web.bean.utils;

import java.sql.SQLException;
import java.util.Calendar;

import javax.transaction.Transactional;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import lcsb.mapviewer.persist.DbUtils;
import lcsb.mapviewer.persist.dao.cache.UploadedFileEntryDao;
import lcsb.mapviewer.services.interfaces.IMinervaJobService;

@Component
public class DbMaintenanceBean {

  private final transient Logger logger = LogManager.getLogger();

  private transient DbUtils dbUtils;
  private transient UploadedFileEntryDao uploadedFileEntryDao;
  private transient IMinervaJobService minervaJobService;

  @Autowired
  public DbMaintenanceBean(final DbUtils dbUtils, final UploadedFileEntryDao uploadedFileEntryDao, final IMinervaJobService minervaJobService) {
    this.dbUtils = dbUtils;
    this.uploadedFileEntryDao = uploadedFileEntryDao;
    this.minervaJobService = minervaJobService;
  }

  @Transactional
  @Scheduled(cron = "0 15 2 * * ?")
  public void scheduleTaskUsingCronExpression() {
    try {
      logger.info("VACUUM started");
      dbUtils.vaccuum();
    } catch (SQLException e) {
      logger.error("Problem with vacuum", e);
    } finally {
      logger.info("VACUUM finished");
    }
  }

  @Transactional
  @Scheduled(cron = "0 0 1 * * SAT")
  public void scheduleOrphaFileRemoveTaskUsingCronExpression() {
    try {
      logger.info("Remove orphan files");
      uploadedFileEntryDao.removeOrphans();
    } catch (Exception e) {
      logger.error("Problem with Remove orphan files", e);
    } finally {
      logger.info("Remove orphan finished");
    }
  }

  @Transactional
  @Scheduled(cron = "0 15 1 * * ?")
  public void removeOldJobsExpression() {
    try {
      logger.info("Remove old jobs from db");
      Calendar beforeDate = Calendar.getInstance();
      beforeDate.add(Calendar.MONTH, -1);
      minervaJobService.removeOldJobs(beforeDate);
    } finally {
      logger.info("VACUUM finished");
    }
  }

}
